@@ -1,8 +1,8 @@
-/* $Id: DataEntryWriterFactory.java,v 1.2 2004/08/15 12:39:30 eric Exp $
+/* $Id: DataEntryWriterFactory.java,v 1.4 2005/06/11 13:13:15 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2004 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -84,14 +84,14 @@ public class DataEntryWriterFactory
                             isWar ? "ear" :
                             isZip ? "zip" :
                                     "directory") +
-                           " [" + classPathEntry.getName() + "]" +
+                           " [" + entryName + "]" +
                            (filter    != null ||
                             jarFilter != null ||
                             warFilter != null ||
                             earFilter != null ||
                             zipFilter != null ? " (filtered)" : ""));
 
-        DataEntryWriter writer = new DirectoryWriter(new File(entryName),
+        DataEntryWriter writer = new DirectoryWriter(classPathEntry.getFile(),
                                                      isJar ||
                                                      isWar ||
                                                      isEar ||
