@@ -1,8 +1,8 @@
-/* $Id: FileWordReader.java,v 1.9 2004/08/15 12:39:30 eric Exp $
+/* $Id: FileWordReader.java,v 1.12 2005/06/11 13:13:15 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2004 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -31,19 +31,21 @@ import java.net.URL;
  */
 public class FileWordReader extends WordReader
 {
-    private String           fileName;
+    private String           name;
     private LineNumberReader reader;
 
 
     /**
-     * Creates a new FileWordReader for the given file name.
+     * Creates a new FileWordReader for the given file.
      */
-    public FileWordReader(String fileName) throws IOException
+    public FileWordReader(File file) throws IOException
     {
-        this.fileName = fileName;
-        this.reader   = new LineNumberReader(
-                        new BufferedReader(
-                        new FileReader(fileName)));
+        super(file.getParentFile());
+
+        this.name   = file.getPath();
+        this.reader = new LineNumberReader(
+                      new BufferedReader(
+                      new FileReader(file)));
     }
 
 
@@ -52,23 +54,12 @@ public class FileWordReader extends WordReader
      */
     public FileWordReader(URL url) throws IOException
     {
-        this.fileName = url.getPath();
-        this.reader   = new LineNumberReader(
-                        new BufferedReader(
-                        new InputStreamReader(
-                        url.openStream())));
-    }
+        super(null);
 
-
-    /**
-     * Closes the FileWordReader.
-     */
-    public void close() throws IOException
-    {
-        if (reader != null)
-        {
-            reader.close();
-        }
+        this.name    = url.toString();
+        this.reader  = new LineNumberReader(
+                       new BufferedReader(
+                       new InputStreamReader(url.openStream())));
     }
 
 
@@ -82,6 +73,17 @@ public class FileWordReader extends WordReader
 
     protected String lineLocationDescription()
     {
-        return "line " + reader.getLineNumber() + " of file '" + fileName + "'";
+        return "line " + reader.getLineNumber() + " of file '" + name + "'";
+    }
+
+
+    public void close() throws IOException
+    {
+        super.close();
+
+        if (reader != null)
+        {
+            reader.close();
+        }
     }
 }
