@@ -1,14 +1,15 @@
-/* $Id: ProGuard.java,v 1.84 2004/12/11 16:35:23 eric Exp $
+/* $Id: ProGuard.java,v 1.101 2005/10/22 11:53:39 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2004 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
  * Software Foundation; either version 2 of the License, or (at your option)
  * any later version.
  *
+ *
  * This program is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
@@ -20,21 +21,20 @@
  */
 package proguard;
 
-import proguard.classfile.*;
-import proguard.classfile.attribute.*;
-import proguard.classfile.editor.*;
-import proguard.classfile.instruction.*;
+import proguard.classfile.ClassPool;
+import proguard.classfile.attribute.AllAttrInfoVisitor;
+import proguard.classfile.editor.ConstantPoolSorter;
+import proguard.classfile.instruction.AllInstructionVisitor;
 import proguard.classfile.util.*;
 import proguard.classfile.visitor.*;
 import proguard.io.*;
-import proguard.obfuscate.*;
-import proguard.optimize.*;
-import proguard.optimize.evaluation.*;
-import proguard.optimize.peephole.*;
-import proguard.shrink.*;
+import proguard.obfuscate.Obfuscator;
+import proguard.optimize.Optimizer;
+import proguard.shrink.Shrinker;
+import proguard.util.ClassNameListMatcher;
 
 import java.io.*;
-
+import java.util.*;
 
 /**
  * Tool for shrinking, optimizing, and obfuscating Java class files.
@@ -43,7 +43,7 @@ import java.io.*;
  */
 public class ProGuard
 {
-    public static final String VERSION = "ProGuard, version 3.2";
+    public static final String VERSION = "ProGuard, version 3.4";
 
     private Configuration configuration;
     private ClassPool     programClassPool = new ClassPool();
@@ -67,8 +67,16 @@ public class ProGuard
     {
         System.out.println(VERSION);
 
+        GPL.check();
+
         readInput();
 
+        // The defaultPackage option implies the allowAccessModification option.
+        if (configuration.defaultPackage != null)
+        {
+            configuration.allowAccessModification = true;
+        }
+
         if (configuration.shrink   ||
             configuration.optimize ||
             configuration.obfuscate)
@@ -93,7 +101,9 @@ public class ProGuard
             // Shrink again, if we may.
             if (configuration.shrink)
             {
-                configuration.printUsage = null;
+                // Don't print any usage this time around.
+                configuration.printUsage       = null;
+                configuration.whyAreYouKeeping = null;
 
                 shrink();
             }
@@ -157,12 +167,6 @@ public class ProGuard
                       configuration.libraryJars,
                       createDataEntryClassPoolFiller(true));
         }
-
-        // The defaultPackage option implies the allowAccessModification option.
-        if (configuration.defaultPackage != null)
-        {
-            configuration.allowAccessModification = true;
-        }
     }
 
 
@@ -185,6 +189,7 @@ public class ProGuard
             new ClassFileReader(isLibrary,
                                 configuration.skipNonPublicLibraryClasses,
                                 configuration.skipNonPublicLibraryClassMembers,
+                                configuration.note,
             new ClassPoolFiller(classPool, configuration.note)));
     }
 
@@ -241,7 +246,7 @@ public class ProGuard
 
             // Create the data entry pump.
             DirectoryPump directoryPump =
-                new DirectoryPump(new File(classPathEntry.getName()));
+                new DirectoryPump(classPathEntry.getFile());
 
             // Pump the data entries into the reader.
             directoryPump.pumpDataEntries(reader);
@@ -258,6 +263,13 @@ public class ProGuard
      */
     private void initialize() throws IOException
     {
+        if (configuration.verbose)
+        {
+            System.out.println("Initializing...");
+        }
+
+        int originalLibraryClassPoolSize = libraryClassPool.size();
+
         // Initialize the class hierarchy for program class files.
         ClassFileHierarchyInitializer classFileHierarchyInitializer =
             new ClassFileHierarchyInitializer(programClassPool,
@@ -266,7 +278,7 @@ public class ProGuard
 
         programClassPool.classFilesAccept(classFileHierarchyInitializer);
 
-        // Initialize the rest of the class hierarchy.
+        // Initialize the class hierarchy for library class files.
         ClassFileHierarchyInitializer classFileHierarchyInitializer2 =
             new ClassFileHierarchyInitializer(programClassPool,
                                               libraryClassPool,
@@ -274,16 +286,48 @@ public class ProGuard
 
         libraryClassPool.classFilesAccept(classFileHierarchyInitializer2);
 
-        // Initialize the other class references.
+        // Initialize the Class.forName and .class references.
+        ClassFileClassForNameReferenceInitializer classFileClassForNameReferenceInitializer =
+            new ClassFileClassForNameReferenceInitializer(programClassPool,
+                                                          libraryClassPool,
+                                                          configuration.note,
+                                                          createNoteExceptionMatcher(configuration.keep));
+
+        programClassPool.classFilesAccept(
+            new AllMethodVisitor(
+            new AllAttrInfoVisitor(
+            new AllInstructionVisitor(classFileClassForNameReferenceInitializer))));
+
+        // Initialize the class references from program class members and attributes.
         ClassFileReferenceInitializer classFileReferenceInitializer =
             new ClassFileReferenceInitializer(programClassPool,
                                               libraryClassPool,
-                                              configuration.warn,
-                                              configuration.note);
+                                              configuration.warn);
 
         programClassPool.classFilesAccept(classFileReferenceInitializer);
 
-        int noteCount = classFileReferenceInitializer.getNoteCount();
+        // Reinitialize the library class pool with only those library classes
+        // whose hierarchies are referenced by the program classes.
+        ClassPool newLibraryClassPool = new ClassPool();
+        programClassPool.classFilesAccept(
+            new AllCpInfoVisitor(
+            new ReferencedClassFileVisitor(
+            new LibraryClassFileFilter(
+            new ClassFileHierarchyTraveler(true, true, true, false,
+            new LibraryClassFileFilter(
+            new ClassPoolFiller(newLibraryClassPool, false)))))));
+
+        libraryClassPool = newLibraryClassPool;
+
+        // Initialize the class references from library class members.
+        ClassFileReferenceInitializer classFileReferenceInitializer2 =
+            new ClassFileReferenceInitializer(programClassPool,
+                                              libraryClassPool,
+                                              false);
+
+        libraryClassPool.classFilesAccept(classFileReferenceInitializer2);
+
+        int noteCount = classFileClassForNameReferenceInitializer.getNoteCount();
         if (noteCount > 0)
         {
             System.err.println("Note: there were " + noteCount +
@@ -322,27 +366,50 @@ public class ProGuard
         // Discard unused library classes.
         if (configuration.verbose)
         {
-                    System.out.println("Removing unused library classes...");
-                    System.out.println("    Original number of library classes: " + libraryClassPool.size());
+            System.out.println("Removed unused library classes...");
+            System.out.println("  Original number of library classes: " + originalLibraryClassPoolSize);
+            System.out.println("  Final number of library classes:    " + libraryClassPool.size());
         }
+    }
 
-        // Reinitialize the library class pool with only those library classes
-        // whose hierarchies are referenced by the program classes.
-        ClassPool newLibraryClassPool = new ClassPool();
-        programClassPool.classFilesAccept(
-            new AllCpInfoVisitor(
-            new ReferencedClassFileVisitor(
-            new LibraryClassFileFilter(
-            new ClassFileHierarchyTraveler(true, true, true, false,
-            new LibraryClassFileFilter(
-            new ClassPoolFiller(newLibraryClassPool, false)))))));
 
-        libraryClassPool = newLibraryClassPool;
-
-        if (configuration.verbose)
+    /**
+     * Extracts a list of exceptions for which not to print notes, from the
+     * keep configuration.
+     */
+    private ClassNameListMatcher createNoteExceptionMatcher(List noteExceptions)
+    {
+        if (noteExceptions != null)
         {
-            System.out.println("    Final number of library classes:    " + libraryClassPool.size());
+            List noteExceptionNames = new ArrayList(noteExceptions.size());
+            for (int index = 0; index < noteExceptions.size(); index++)
+            {
+                ClassSpecification classSpecification = (ClassSpecification)noteExceptions.get(index);
+                if (classSpecification.markClassFiles)
+                {
+                    // If the class itself is being kept, it's ok.
+                    String className = classSpecification.className;
+                    if (className != null)
+                    {
+                        noteExceptionNames.add(className);
+                    }
+
+                    // If all of its extensions are being kept, it's ok too.
+                    String extendsClassName = classSpecification.extendsClassName;
+                    if (extendsClassName != null)
+                    {
+                        noteExceptionNames.add(extendsClassName);
+                    }
+                }
+            }
+
+            if (noteExceptionNames.size() > 0)
+            {
+                return new ClassNameListMatcher(noteExceptionNames);
+            }
         }
+
+        return null;
     }
 
 
@@ -363,7 +430,7 @@ public class ProGuard
             throw new IOException("You have to specify '-keep' options for the shrinking step.");
         }
 
-        PrintStream ps = configuration.printSeeds.length() > 0 ?
+        PrintStream ps = isFile(configuration.printSeeds) ?
             new PrintStream(new BufferedOutputStream(new FileOutputStream(configuration.printSeeds))) :
             System.out;
 
@@ -394,6 +461,21 @@ public class ProGuard
         if (configuration.verbose)
         {
             System.out.println("Shrinking...");
+
+            // We'll print out some explanation, if requested.
+            if (configuration.whyAreYouKeeping != null)
+            {
+                System.out.println("Explaining why classes and class members are being kept...");
+            }
+
+            // We'll print out the usage, if requested.
+            if (configuration.printUsage != null)
+            {
+                System.out.println("Printing usage" +
+                                   (isFile(configuration.printUsage) ?
+                                       " to [" + configuration.printUsage.getAbsolutePath() + "]" :
+                                       "..."));
+            }
         }
 
         // Check if we have at least some keep commands.
@@ -402,77 +484,24 @@ public class ProGuard
             throw new IOException("You have to specify '-keep' options for the shrinking step.");
         }
 
-        // Clean up any old visitor info.
-        ClassFileCleaner classFileCleaner = new ClassFileCleaner();
-        programClassPool.classFilesAccept(classFileCleaner);
-        libraryClassPool.classFilesAccept(classFileCleaner);
+        int originalProgramClassPoolSize = programClassPool.size();
 
-        // Create a visitor for marking the seeds.
-        UsageMarker usageMarker = new UsageMarker();
-        ClassPoolVisitor classPoolvisitor =
-            ClassSpecificationVisitorFactory.createClassPoolVisitor(configuration.keep,
-                                                                    usageMarker,
-                                                                    usageMarker);
-        // Mark the seeds.
-        programClassPool.accept(classPoolvisitor);
-        libraryClassPool.accept(classPoolvisitor);
-
-        // Mark interfaces that have to be kept.
-        programClassPool.classFilesAccept(new InterfaceUsageMarker());
-
-        // Mark the inner class information that has to be kept.
-        programClassPool.classFilesAccept(new InnerUsageMarker());
-
-        if (configuration.printUsage != null)
-        {
-            if (configuration.verbose)
-            {
-                System.out.println("Printing usage" +
-                                   (configuration.printUsage.length() > 0 ?
-                                       " to [" + configuration.printUsage + "]" :
-                                       "..."));
-            }
-
-            PrintStream ps = configuration.printUsage.length() > 0 ?
-                new PrintStream(new BufferedOutputStream(new FileOutputStream(configuration.printUsage))) :
-                System.out;
-
-            // Print out items that will be removed.
-            programClassPool.classFilesAcceptAlphabetically(new UsagePrinter(true, ps));
-
-            if (ps != System.out)
-            {
-                ps.close();
-            }
-        }
-
-        // Discard unused program classes.
-        if (configuration.verbose)
-        {
-            System.out.println("Removing unused program classes and class elements...");
-            System.out.println("    Original number of program classes: " + programClassPool.size());
-        }
-
-        ClassPool newProgramClassPool = new ClassPool();
-        programClassPool.classFilesAccept(
-            new UsedClassFileFilter(
-            new MultiClassFileVisitor(
-            new ClassFileVisitor[] {
-                new ClassFileShrinker(1024),
-                new ClassPoolFiller(newProgramClassPool, false)
-            })));
-        programClassPool = newProgramClassPool;
-
-        if (configuration.verbose)
-        {
-            System.out.println("    Final number of program classes:    " + programClassPool.size());
-        }
+        // Perform the actual shrinking.
+        programClassPool = new Shrinker(configuration).execute(programClassPool, libraryClassPool);
 
         // Check if we have at least some output class files.
-        if (programClassPool.size() == 0)
+        int newProgramClassPoolSize = programClassPool.size();
+        if (newProgramClassPoolSize == 0)
         {
             throw new IOException("The output jar is empty. Did you specify the proper '-keep' options?");
         }
+
+        if (configuration.verbose)
+        {
+            System.out.println("Removed unused program classes and class elements...");
+            System.out.println("  Original number of program classes: " + originalProgramClassPoolSize);
+            System.out.println("  Final number of program classes:    " + newProgramClassPoolSize);
+        }
     }
 
 
@@ -486,107 +515,17 @@ public class ProGuard
             System.out.println("Optimizing...");
         }
 
-        // Clean up any old visitor info.
-        ClassFileCleaner classFileCleaner = new ClassFileCleaner();
-        programClassPool.classFilesAccept(classFileCleaner);
-        libraryClassPool.classFilesAccept(classFileCleaner);
-
         // Check if we have at least some keep commands.
-        if (configuration.keep == null)
+        if (configuration.keep         == null &&
+            configuration.keepNames    == null &&
+            configuration.applyMapping == null &&
+            configuration.printMapping == null)
         {
             throw new IOException("You have to specify '-keep' options for the optimization step.");
         }
 
-        // Create a visitor for marking the seeds.
-        KeepMarker keepMarker = new KeepMarker();
-        ClassPoolVisitor classPoolvisitor =
-            ClassSpecificationVisitorFactory.createClassPoolVisitor(configuration.keep,
-                                                                    keepMarker,
-                                                                    keepMarker);
-
-        // Mark the seeds.
-        programClassPool.accept(classPoolvisitor);
-        libraryClassPool.accept(classPoolvisitor);
-
-        // Make class files and methods final, as far as possible.
-        programClassPool.classFilesAccept(new ClassFileFinalizer());
-
-        // Mark all fields that are write-only.
-        programClassPool.classFilesAccept(
-            new AllMethodVisitor(
-            new AllAttrInfoVisitor(
-            new AllInstructionVisitor(
-            new WriteOnlyFieldMarker()))));
-
-        if (configuration.assumeNoSideEffects != null)
-        {
-            // Create a visitor for marking methods that don't have any side effects.
-            NoSideEffectMethodMarker noSideEffectMethodMarker = new NoSideEffectMethodMarker();
-            ClassPoolVisitor noClassPoolvisitor =
-                ClassSpecificationVisitorFactory.createClassPoolVisitor(configuration.assumeNoSideEffects,
-                                                                        null,
-                                                                        noSideEffectMethodMarker);
-
-            // Mark the seeds.
-            programClassPool.accept(noClassPoolvisitor);
-            libraryClassPool.accept(noClassPoolvisitor);
-        }
-
-        // Mark all methods that have side effects.
-        programClassPool.accept(new SideEffectMethodMarker());
-
-        // Mark all interfaces that have single implementations.
-        programClassPool.classFilesAccept(new SingleImplementationMarker(configuration.allowAccessModification));
-
-        // Inline interfaces with single implementations.
-        // First update the references to classes and class members.
-        // Then update the class member descriptors.
-        programClassPool.classFilesAccept(new AllMethodVisitor(
-                                          new AllAttrInfoVisitor(
-                                          new SingleImplementationInliner())));
-        programClassPool.classFilesAccept(new AllMemberInfoVisitor(
-                                          new SingleImplementationInliner()));
-
-        // Perform partial evaluation.
-        programClassPool.classFilesAccept(new AllMethodVisitor(
-                                          new PartialEvaluator()));
-
-        // Create a branch target marker and a code attribute editor that can
-        // be reused for all code attributes.
-        BranchTargetFinder branchTargetFinder = new BranchTargetFinder(1024);
-        CodeAttrInfoEditor codeAttrInfoEditor = new CodeAttrInfoEditor(1024);
-
-        // Visit all code attributes.
-        // First let the branch marker mark all branch targets.
-        // Then perform peephole optimisations on the instructions:
-        // - Remove push/pop instruction pairs.
-        // - Remove load/store instruction pairs.
-        // - Replace store/load instruction pairs by dup/store instructions.
-        // - Replace branches to return instructions by return instructions.
-        // - Remove nop instructions.
-        // - Inline simple getters and setters.
-        // Finally apply all changes to the code.
-        programClassPool.classFilesAccept(
-            new AllMethodVisitor(
-            new AllAttrInfoVisitor(
-            new MultiAttrInfoVisitor(
-            new AttrInfoVisitor[]
-            {
-                branchTargetFinder,
-                new CodeAttrInfoEditorResetter(codeAttrInfoEditor),
-                new AllInstructionVisitor(
-                new MultiInstructionVisitor(
-                new InstructionVisitor[]
-                {
-                    new PushPopRemover(branchTargetFinder, codeAttrInfoEditor),
-                    new LoadStoreRemover(branchTargetFinder, codeAttrInfoEditor),
-                    new StoreLoadReplacer(branchTargetFinder, codeAttrInfoEditor),
-                    new GotoReturnReplacer(codeAttrInfoEditor),
-                    new NopRemover(codeAttrInfoEditor),
-                    new GetterSetterInliner(codeAttrInfoEditor, configuration.allowAccessModification),
-                })),
-                codeAttrInfoEditor
-            }))));
+        // Perform the actual optimization.
+        new Optimizer(configuration).execute(programClassPool, libraryClassPool);
     }
 
 
@@ -598,130 +537,25 @@ public class ProGuard
         if (configuration.verbose)
         {
             System.out.println("Obfuscating...");
-        }
 
-        // Check if we have at least some keep commands.
-        if (configuration.keep      == null &&
-            configuration.keepNames == null)
-        {
-            throw new IOException("You have to specify '-keep' options for the obfuscation step.");
-        }
-
-        // Clean up any old visitor info.
-        ClassFileCleaner classFileCleaner = new ClassFileCleaner();
-        programClassPool.classFilesAccept(classFileCleaner);
-        libraryClassPool.classFilesAccept(classFileCleaner);
-
-        // Link all class members that should get the same names.
-        programClassPool.classFilesAccept(new BottomClassFileFilter(
-                                          new MemberInfoLinker()));
-
-        // Create a visitor for marking the seeds.
-        NameMarker nameMarker = new NameMarker();
-        ClassPoolVisitor classPoolvisitor =
-            new MultiClassPoolVisitor(new ClassPoolVisitor[]
+            // We'll apply a mapping, if requested.
+            if (configuration.applyMapping != null)
             {
-                ClassSpecificationVisitorFactory.createClassPoolVisitor(configuration.keep,
-                                                                        nameMarker,
-                                                                        nameMarker),
-                ClassSpecificationVisitorFactory.createClassPoolVisitor(configuration.keepNames,
-                                                                        nameMarker,
-                                                                        nameMarker)
-            });
-
-        // Mark the seeds.
-        programClassPool.accept(classPoolvisitor);
-        libraryClassPool.accept(classPoolvisitor);
-
-        // Apply the mapping, if one has been specified.
-        if (configuration.applyMapping != null)
-        {
-            if (configuration.verbose)
-            {
-                System.out.println("Applying mapping [" + configuration.applyMapping + "]");
+                System.out.println("Applying mapping [" + configuration.applyMapping.getAbsolutePath() + "]");
             }
 
-            MappingReader    reader = new MappingReader(configuration.applyMapping);
-            MappingProcessor keeper =
-                new MultiMappingProcessor(new MappingProcessor[]
-                {
-                    new MappingKeeper(programClassPool),
-                    new MappingKeeper(libraryClassPool),
-                });
-
-            reader.pump(keeper);
-        }
-
-        // Mark attributes that have to be kept.
-        AttributeUsageMarker attributeUsageMarker = new AttributeUsageMarker();
-        if (configuration.keepAttributes != null)
-        {
-            if (configuration.keepAttributes.size() != 0)
-            {
-                attributeUsageMarker.setKeepAttributes(configuration.keepAttributes);
-            }
-            else
-            {
-                attributeUsageMarker.setKeepAllAttributes();
-            }
-        }
-        programClassPool.classFilesAccept(attributeUsageMarker);
-
-        // Remove the attributes that can be discarded.
-        programClassPool.classFilesAccept(new AttributeShrinker());
-
-        if (configuration.verbose)
-        {
-            System.out.println("Renaming program classes and class elements...");
-        }
-
-        // Come up with new names for all class files.
-        programClassPool.classFilesAccept(new ClassFileObfuscator(programClassPool,
-                                                                  configuration.defaultPackage,
-                                                                  configuration.useMixedCaseClassNames));
-
-        // Come up with new names for all class members.
-        programClassPool.classFilesAccept(new BottomClassFileFilter(
-                                          new MemberInfoObfuscator(configuration.overloadAggressively,
-                                                                   configuration.obfuscationDictionary)));
-
-        // Print out the mapping, if requested.
-        if (configuration.printMapping != null)
-        {
-            if (configuration.verbose)
+            // We'll print out the mapping, if requested.
+            if (configuration.printMapping != null)
             {
                 System.out.println("Printing mapping" +
-                                   (configuration.printMapping.length() > 0 ?
-                                       " to [" + configuration.printMapping + "]" :
+                                   (isFile(configuration.printMapping) ?
+                                       " to [" + configuration.printMapping.getAbsolutePath() + "]" :
                                        "..."));
             }
-
-            PrintStream ps = configuration.printMapping.length() > 0 ?
-                new PrintStream(new BufferedOutputStream(new FileOutputStream(configuration.printMapping))) :
-                System.out;
-
-            // Print out items that will be removed.
-            programClassPool.classFilesAcceptAlphabetically(new MappingPrinter(ps));
-
-            if (ps != System.out)
-            {
-                ps.close();
-            }
         }
 
-        // Actually apply these new names.
-        programClassPool.classFilesAccept(new ClassFileRenamer(configuration.defaultPackage != null,
-                                                               configuration.newSourceFileAttribute));
-
-        // Mark NameAndType constant pool entries that have to be kept
-        // and remove the other ones.
-        programClassPool.classFilesAccept(new NameAndTypeUsageMarker());
-        programClassPool.classFilesAccept(new NameAndTypeShrinker(1024));
-
-        // Mark Utf8 constant pool entries that have to be kept
-        // and remove the other ones.
-        programClassPool.classFilesAccept(new Utf8UsageMarker());
-        programClassPool.classFilesAccept(new Utf8Shrinker(1024));
+        // Perform the actual obfuscation.
+        new Obfuscator(configuration).execute(programClassPool, libraryClassPool);
     }
 
 
@@ -730,6 +564,7 @@ public class ProGuard
      */
     private void sortConstantPools()
     {
+        // TODO: Avoid duplicate constant pool entries.
         programClassPool.classFilesAccept(new ConstantPoolSorter(1024));
     }
 
@@ -778,7 +613,7 @@ public class ProGuard
                     ClassPathEntry otherEntry = programJars.get(inIndex);
 
                     if (!otherEntry.isOutput() &&
-                        entry.getName().equals(otherEntry.getName()))
+                        entry.getFile().equals(otherEntry.getFile()))
                     {
                         throw new IOException("The output jar [" + entry.getName() +
                                               "] must be different from all input jars.");
@@ -849,7 +684,7 @@ public class ProGuard
                                     new DataEntryCopier(writer));
 
             // Read and handle the specified input entries.
-            readInput("Copying resources from program ",
+            readInput("  Copying resources from program ",
                       classPath,
                       fromInputIndex,
                       fromOutputIndex,
@@ -873,24 +708,34 @@ public class ProGuard
         if (configuration.verbose)
         {
             System.out.println("Printing classes" +
-                               (configuration.dump.length() > 0 ?
-                                   " to [" + configuration.dump + "]" :
+                               (isFile(configuration.dump) ?
+                                   " to [" + configuration.dump.getAbsolutePath() + "]" :
                                    "..."));
         }
 
-        PrintStream ps = configuration.dump.length() > 0 ?
+        PrintStream ps = isFile(configuration.dump) ?
             new PrintStream(new BufferedOutputStream(new FileOutputStream(configuration.dump))) :
             System.out;
 
         programClassPool.classFilesAccept(new ClassFilePrinter(ps));
 
-        if (configuration.dump.length() > 0)
+        if (isFile(configuration.dump))
         {
             ps.close();
         }
     }
 
 
+    /**
+     * Returns whether the given file is actually a file, or just a placeholder
+     * for the standard output.
+     */
+    private boolean isFile(File file)
+    {
+        return file.getPath().length() > 0;
+    }
+
+
     /**
      * The main method for ProGuard.
      */
@@ -898,6 +743,7 @@ public class ProGuard
     {
         if (args.length == 0)
         {
+            System.out.println(VERSION);
             System.out.println("Usage: java proguard.ProGuard [options ...]");
             System.exit(1);
         }
@@ -909,11 +755,18 @@ public class ProGuard
         {
             // Parse the options specified in the command line arguments.
             ConfigurationParser parser = new ConfigurationParser(args);
-            parser.parse(configuration);
 
-            // Execute ProGuard with these options.
-            ProGuard proGuard = new ProGuard(configuration);
-            proGuard.execute();
+            try
+            {
+                parser.parse(configuration);
+
+                // Execute ProGuard with these options.
+                new ProGuard(configuration).execute();
+            }
+            finally
+            {
+                parser.close();
+            }
         }
         catch (Exception ex)
         {
