@@ -1,8 +1,8 @@
-/* $Id: ConstantPoolEditor.java,v 1.7 2004/11/20 15:41:24 eric Exp $
+/* $Id: ConstantPoolEditor.java,v 1.10 2005/06/11 13:21:35 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2004 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -29,6 +29,9 @@ import proguard.classfile.*;
  */
 public class ConstantPoolEditor
 {
+    private static final boolean DEBUG = false;
+
+
     /**
      * Finds or creates a StringCpInfo constant pool entry with the given value,
      * in the given class file.
@@ -65,6 +68,24 @@ public class ConstantPoolEditor
     }
 
 
+    /**
+     * Finds or creates a FieldrefCpInfo constant pool entry for the given
+     * class and field, in the given class file.
+     * @return the constant pool index of the FieldrefCpInfo.
+     */
+    public int addFieldrefCpInfo(ProgramClassFile programClassFile,
+                                 ClassFile        referencedClassFile,
+                                 MemberInfo       referencedMemberInfo)
+    {
+        return addFieldrefCpInfo(programClassFile,
+                                 referencedClassFile.getName(),
+                                 referencedMemberInfo.getName(referencedClassFile),
+                                 referencedMemberInfo.getDescriptor(referencedClassFile),
+                                 referencedClassFile,
+                                 referencedMemberInfo);
+    }
+
+
     /**
      * Finds or creates a FieldrefCpInfo constant pool entry with the given
      * class name, field name, and descriptor, in the given class file.
@@ -75,15 +96,13 @@ public class ConstantPoolEditor
                                  String           name,
                                  String           descriptor,
                                  ClassFile        referencedClassFile,
-                                 MemberInfo       referencedMemberInfo,
-                                 ClassFile[]      referencedClassFiles)
+                                 MemberInfo       referencedMemberInfo)
     {
         return addFieldrefCpInfo(programClassFile,
                                  className,
                                  addNameAndTypeCpInfo(programClassFile,
                                                       name,
-                                                      descriptor,
-                                                      referencedClassFiles),
+                                                      descriptor),
                                  referencedClassFile,
                                  referencedMemberInfo);
     }
@@ -121,15 +140,13 @@ public class ConstantPoolEditor
                                  String           name,
                                  String           descriptor,
                                  ClassFile        referencedClassFile,
-                                 MemberInfo       referencedMemberInfo,
-                                 ClassFile[]      referencedClassFiles)
+                                 MemberInfo       referencedMemberInfo)
     {
         return addFieldrefCpInfo(programClassFile,
                                  classIndex,
                                  addNameAndTypeCpInfo(programClassFile,
                                                       name,
-                                                      descriptor,
-                                                      referencedClassFiles),
+                                                      descriptor),
                                  referencedClassFile,
                                  referencedMemberInfo);
     }
@@ -175,6 +192,148 @@ public class ConstantPoolEditor
     }
 
 
+    /**
+     * Finds or creates a InterfaceMethodrefCpInfo constant pool entry with the
+     * given class name, method name, and descriptor, in the given class file.
+     * @return the constant pool index of the InterfaceMethodrefCpInfo.
+     */
+    public int addInterfaceMethodrefCpInfo(ProgramClassFile programClassFile,
+                                           String           className,
+                                           String           name,
+                                           String           descriptor,
+                                           ClassFile        referencedClassFile,
+                                           MemberInfo       referencedMemberInfo)
+    {
+        return addInterfaceMethodrefCpInfo(programClassFile,
+                                           className,
+                                           addNameAndTypeCpInfo(programClassFile,
+                                                                name,
+                                                                descriptor),
+                                                                referencedClassFile,
+                                                                referencedMemberInfo);
+    }
+
+
+    /**
+     * Finds or creates a InterfaceMethodrefCpInfo constant pool entry with the
+     * given class name, method name, and descriptor, in the given class file.
+     * @return the constant pool index of the InterfaceMethodrefCpInfo.
+     */
+    public int addInterfaceMethodrefCpInfo(ProgramClassFile programClassFile,
+                                           String           className,
+                                           int              nameAndTypeIndex,
+                                           ClassFile        referencedClassFile,
+                                           MemberInfo       referencedMemberInfo)
+    {
+        return addInterfaceMethodrefCpInfo(programClassFile,
+                                           addClassCpInfo(programClassFile,
+                                                          className,
+                                                          referencedClassFile),
+                                                          nameAndTypeIndex,
+                                                          referencedClassFile,
+                                                          referencedMemberInfo);
+    }
+
+
+    /**
+     * Finds or creates a InterfaceMethodrefCpInfo constant pool entry for the
+     * given class and method, in the given class file.
+     * @return the constant pool index of the InterfaceMethodrefCpInfo.
+     */
+    public int addInterfaceMethodrefCpInfo(ProgramClassFile programClassFile,
+                                           ClassFile        referencedClassFile,
+                                           MemberInfo       referencedMemberInfo)
+    {
+        return addInterfaceMethodrefCpInfo(programClassFile,
+                                           referencedClassFile.getName(),
+                                           referencedMemberInfo.getName(referencedClassFile),
+                                           referencedMemberInfo.getDescriptor(referencedClassFile),
+                                           referencedClassFile,
+                                           referencedMemberInfo);
+    }
+
+
+    /**
+     * Finds or creates a InterfaceMethodrefCpInfo constant pool entry with the
+     * given class constant pool entry index, method name, and descriptor, in
+     * the given class file.
+     * @return the constant pool index of the InterfaceMethodrefCpInfo.
+     */
+    public int addInterfaceMethodrefCpInfo(ProgramClassFile programClassFile,
+                                           int              classIndex,
+                                           String           name,
+                                           String           descriptor,
+                                           ClassFile        referencedClassFile,
+                                           MemberInfo       referencedMemberInfo)
+    {
+        return addInterfaceMethodrefCpInfo(programClassFile,
+                                           classIndex,
+                                           addNameAndTypeCpInfo(programClassFile,
+                                                                name,
+                                                                descriptor),
+                                           referencedClassFile,
+                                           referencedMemberInfo);
+    }
+
+
+    /**
+     * Finds or creates a InterfaceMethodrefCpInfo constant pool entry with the
+     * given class constant pool entry index and name and type constant pool
+     * entry index the given class file.
+     * @return the constant pool index of the InterfaceMethodrefCpInfo.
+     */
+    public int addInterfaceMethodrefCpInfo(ProgramClassFile programClassFile,
+                                           int              classIndex,
+                                           int              nameAndTypeIndex,
+                                           ClassFile        referencedClassFile,
+                                           MemberInfo       referencedMemberInfo)
+    {
+        CpInfo[] constantPool      = programClassFile.constantPool;
+        int      constantPoolCount = programClassFile.u2constantPoolCount;
+
+        // Check if the entry already exists.
+        for (int index = 1; index < constantPoolCount; index++)
+        {
+            CpInfo cpInfo = constantPool[index];
+
+            if (cpInfo != null &&
+                            cpInfo.getTag() == ClassConstants.CONSTANT_InterfaceMethodref)
+            {
+                InterfaceMethodrefCpInfo methodrefCpInfo = (InterfaceMethodrefCpInfo)cpInfo;
+                if (methodrefCpInfo.u2classIndex       == classIndex &&
+                                methodrefCpInfo.u2nameAndTypeIndex == nameAndTypeIndex)
+                {
+                    return index;
+                }
+            }
+        }
+
+        return addCpInfo(programClassFile,
+                         new InterfaceMethodrefCpInfo(classIndex,
+                                                      nameAndTypeIndex,
+                                                      referencedClassFile,
+                                                      referencedMemberInfo));
+    }
+
+
+    /**
+     * Finds or creates a MethodrefCpInfo constant pool entry for the given
+     * class and method, in the given class file.
+     * @return the constant pool index of the MethodrefCpInfo.
+     */
+    public int addMethodrefCpInfo(ProgramClassFile programClassFile,
+                                  ClassFile        referencedClassFile,
+                                  MemberInfo       referencedMemberInfo)
+    {
+        return addMethodrefCpInfo(programClassFile,
+                                  referencedClassFile.getName(),
+                                  referencedMemberInfo.getName(referencedClassFile),
+                                  referencedMemberInfo.getDescriptor(referencedClassFile),
+                                  referencedClassFile,
+                                  referencedMemberInfo);
+    }
+
+
     /**
      * Finds or creates a MethodrefCpInfo constant pool entry with the given
      * class name, method name, and descriptor, in the given class file.
@@ -185,15 +344,13 @@ public class ConstantPoolEditor
                                   String           name,
                                   String           descriptor,
                                   ClassFile        referencedClassFile,
-                                  MemberInfo       referencedMemberInfo,
-                                  ClassFile[]      referencedClassFiles)
+                                  MemberInfo       referencedMemberInfo)
     {
         return addMethodrefCpInfo(programClassFile,
                                   className,
                                   addNameAndTypeCpInfo(programClassFile,
                                                        name,
-                                                       descriptor,
-                                                       referencedClassFiles),
+                                                       descriptor),
                                   referencedClassFile,
                                   referencedMemberInfo);
     }
@@ -231,15 +388,13 @@ public class ConstantPoolEditor
                                   String           name,
                                   String           descriptor,
                                   ClassFile        referencedClassFile,
-                                  MemberInfo       referencedMemberInfo,
-                                  ClassFile[]      referencedClassFiles)
+                                  MemberInfo       referencedMemberInfo)
     {
         return addMethodrefCpInfo(programClassFile,
                                   classIndex,
                                   addNameAndTypeCpInfo(programClassFile,
                                                        name,
-                                                       descriptor,
-                                                       referencedClassFiles),
+                                                       descriptor),
                                   referencedClassFile,
                                   referencedMemberInfo);
     }
@@ -286,112 +441,16 @@ public class ConstantPoolEditor
 
 
     /**
-     * Finds or creates a InterfaceMethodrefCpInfo constant pool entry with the
-     * given class name, method name, and descriptor, in the given class file.
-     * @return the constant pool index of the InterfaceMethodrefCpInfo.
+     * Finds or creates a ClassCpInfo constant pool entry for the given class,
+     * in the given class file.
+     * @return the constant pool index of the ClassCpInfo.
      */
-    public int addInterfaceMethodrefCpInfo(ProgramClassFile programClassFile,
-                                           String           className,
-                                           String           name,
-                                           String           descriptor,
-                                           ClassFile        referencedClassFile,
-                                           MemberInfo       referencedMemberInfo,
-                                           ClassFile[]      referencedClassFiles)
+    public int addClassCpInfo(ProgramClassFile programClassFile,
+                              ClassFile        referencedClassFile)
     {
-        return addInterfaceMethodrefCpInfo(programClassFile,
-                                           className,
-                                           addNameAndTypeCpInfo(programClassFile,
-                                                                name,
-                                                                descriptor,
-                                                                referencedClassFiles),
-                                                                referencedClassFile,
-                                                                referencedMemberInfo);
-    }
-
-
-    /**
-     * Finds or creates a InterfaceMethodrefCpInfo constant pool entry with the
-     * given class name, method name, and descriptor, in the given class file.
-     * @return the constant pool index of the InterfaceMethodrefCpInfo.
-     */
-    public int addInterfaceMethodrefCpInfo(ProgramClassFile programClassFile,
-                                           String           className,
-                                           int              nameAndTypeIndex,
-                                           ClassFile        referencedClassFile,
-                                           MemberInfo       referencedMemberInfo)
-    {
-        return addInterfaceMethodrefCpInfo(programClassFile,
-                                           addClassCpInfo(programClassFile,
-                                                          className,
-                                                          referencedClassFile),
-                                                          nameAndTypeIndex,
-                                                          referencedClassFile,
-                                                          referencedMemberInfo);
-    }
-
-
-    /**
-     * Finds or creates a InterfaceMethodrefCpInfo constant pool entry with the
-     * given class constant pool entry index, method name, and descriptor, in
-     * the given class file.
-     * @return the constant pool index of the InterfaceMethodrefCpInfo.
-     */
-    public int addInterfaceMethodrefCpInfo(ProgramClassFile programClassFile,
-                                           int              classIndex,
-                                           String           name,
-                                           String           descriptor,
-                                           ClassFile        referencedClassFile,
-                                           MemberInfo       referencedMemberInfo,
-                                           ClassFile[]      referencedClassFiles)
-    {
-        return addInterfaceMethodrefCpInfo(programClassFile,
-                                           classIndex,
-                                           addNameAndTypeCpInfo(programClassFile,
-                                                                name,
-                                                                descriptor,
-                                                                referencedClassFiles),
-                                                                referencedClassFile,
-                                                                referencedMemberInfo);
-    }
-
-
-    /**
-     * Finds or creates a InterfaceMethodrefCpInfo constant pool entry with the
-     * given class constant pool entry index and name and type constant pool
-     * entry index the given class file.
-     * @return the constant pool index of the InterfaceMethodrefCpInfo.
-     */
-    public int addInterfaceMethodrefCpInfo(ProgramClassFile programClassFile,
-                                           int              classIndex,
-                                           int              nameAndTypeIndex,
-                                           ClassFile        referencedClassFile,
-                                           MemberInfo       referencedMemberInfo)
-    {
-        CpInfo[] constantPool      = programClassFile.constantPool;
-        int      constantPoolCount = programClassFile.u2constantPoolCount;
-
-        // Check if the entry already exists.
-        for (int index = 1; index < constantPoolCount; index++)
-        {
-            CpInfo cpInfo = constantPool[index];
-
-            if (cpInfo != null &&
-                            cpInfo.getTag() == ClassConstants.CONSTANT_InterfaceMethodref)
-            {
-                InterfaceMethodrefCpInfo methodrefCpInfo = (InterfaceMethodrefCpInfo)cpInfo;
-                if (methodrefCpInfo.u2classIndex       == classIndex &&
-                                methodrefCpInfo.u2nameAndTypeIndex == nameAndTypeIndex)
-                {
-                    return index;
-                }
-            }
-        }
-
-        return addCpInfo(programClassFile,
-                         new InterfaceMethodrefCpInfo(classIndex,
-                                                      nameAndTypeIndex,
-                                                      referencedClassFile,
-                                                      referencedMemberInfo));
+        return addClassCpInfo(programClassFile,
+                              referencedClassFile.getName(),
+                              referencedClassFile);
     }
 
 
@@ -404,7 +463,7 @@ public class ConstantPoolEditor
                               String           name,
                               ClassFile        referencedClassFile)
     {
-        CpInfo[] constantPool        = programClassFile.constantPool;
+        CpInfo[] constantPool      = programClassFile.constantPool;
         int      constantPoolCount = programClassFile.u2constantPoolCount;
 
         // Check if the entry already exists.
@@ -438,10 +497,9 @@ public class ConstantPoolEditor
      */
     public int addNameAndTypeCpInfo(ProgramClassFile programClassFile,
                                     String           name,
-                                    String           type,
-                                    ClassFile[]      referencedClassFiles)
+                                    String           type)
     {
-        CpInfo[] constantPool        = programClassFile.constantPool;
+        CpInfo[] constantPool      = programClassFile.constantPool;
         int      constantPoolCount = programClassFile.u2constantPoolCount;
 
         // Check if the entry already exists.
@@ -466,20 +524,19 @@ public class ConstantPoolEditor
 
         return addCpInfo(programClassFile,
                          new NameAndTypeCpInfo(nameIndex,
-                                               descriptorIndex,
-                                               referencedClassFiles));
+                                               descriptorIndex));
     }
 
 
     /**
-     * Finds or creates an Utf8CpInfo constant pool entry for the given string,
+     * Finds or creates a Utf8CpInfo constant pool entry for the given string,
      * in the given class file.
      * @return the constant pool index of the Utf8CpInfo.
      */
     public int addUtf8CpInfo(ProgramClassFile programClassFile,
                              String           string)
     {
-        CpInfo[] constantPool        = programClassFile.constantPool;
+        CpInfo[] constantPool      = programClassFile.constantPool;
         int      constantPoolCount = programClassFile.u2constantPoolCount;
 
         // Check if the entry already exists.
@@ -507,10 +564,10 @@ public class ConstantPoolEditor
      * in the given class file.
      * @return the constant pool index for the added entry.
      */
-    private int addCpInfo(ProgramClassFile programClassFile,
-                          CpInfo           cpInfo)
+    public int addCpInfo(ProgramClassFile programClassFile,
+                         CpInfo           cpInfo)
     {
-        CpInfo[] constantPool        = programClassFile.constantPool;
+        CpInfo[] constantPool      = programClassFile.constantPool;
         int      constantPoolCount = programClassFile.u2constantPoolCount;
 
         // Make sure there is enough space for another constant pool entry.
@@ -523,6 +580,11 @@ public class ConstantPoolEditor
             constantPool = programClassFile.constantPool;
         }
 
+        if (DEBUG)
+        {
+            System.out.println(programClassFile.getName()+": adding ["+cpInfo.getClass().getName()+"] at index "+programClassFile.u2constantPoolCount);
+        }
+
         // Create a new Utf8CpInfo for the given string.
         constantPool[programClassFile.u2constantPoolCount++] = cpInfo;
 
