@@ -1,8 +1,8 @@
-/* $Id: ClassMemberSpecificationDialog.java,v 1.2 2004/08/15 12:39:30 eric Exp $
+/* $Id: ClassMemberSpecificationDialog.java,v 1.5 2005/08/21 19:28:04 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2004 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -133,7 +133,7 @@ class ClassMemberSpecificationDialog extends JDialog
         okButtonConstraints.insets  = new Insets(4, 4, 8, 4);
 
         GridBagConstraints cancelButtonConstraints = new GridBagConstraints();
-        cancelButtonConstraints.gridwidth = GridBagConstraints.REMAINDER;;
+        cancelButtonConstraints.gridwidth = GridBagConstraints.REMAINDER;
         cancelButtonConstraints.weighty   = 1.0;
         cancelButtonConstraints.anchor    = GridBagConstraints.SOUTHEAST;
         cancelButtonConstraints.insets    = okButtonConstraints.insets;
@@ -177,7 +177,9 @@ class ClassMemberSpecificationDialog extends JDialog
         // Create the type panel.
         JPanel typePanel = new JPanel(layout);
         typePanel.setBorder(BorderFactory.createTitledBorder(etchedBorder,
-                                                             GUIResources.getMessage("returnType")));
+                                                             GUIResources.getMessage(isField ?
+                                                                                         "type" :
+                                                                                         "returnType")));
 
         typePanel.add(typeTextField, constraintsLastStretch);
 
@@ -324,19 +326,43 @@ class ClassMemberSpecificationDialog extends JDialog
         String  type      = typeTextField.getText();
         String  arguments = argumentsTextField.getText();
 
-        boolean fullWildcard = name.equals("") ||
-                               type.equals("");
+        if (name.equals("") ||
+            name.equals("*"))
+        {
+            name = null;
+        }
 
-        ClassMemberSpecification classMemberSpecification = fullWildcard ?
-            new ClassMemberSpecification(0,
-                                         0,
-                                         null,
-                                         null) :
-            new ClassMemberSpecification(0,
-                                         0,
-                                         name,
-                                         isField ? ClassUtil.internalType(type) :
-                                                   ClassUtil.internalMethodDescriptor(type, ListUtil.commaSeparatedList(arguments)));
+        if (type.equals("") ||
+            type.equals("*"))
+        {
+            type = null;
+        }
+
+        if (name != null ||
+            type != null)
+        {
+            if (isField)
+            {
+                if (type == null)
+                {
+                    type = ClassConstants.EXTERNAL_TYPE_INT;
+                }
+
+                type = ClassUtil.internalType(type);
+            }
+            else
+            {
+                if (type == null)
+                {
+                    type = ClassConstants.EXTERNAL_TYPE_VOID;
+                }
+
+                type = ClassUtil.internalMethodDescriptor(type, ListUtil.commaSeparatedList(arguments));
+            }
+        }
+
+        ClassMemberSpecification classMemberSpecification =
+            new ClassMemberSpecification(0, 0, name, type);
 
         // Also get the access radio button settings.
         getClassMemberSpecificationRadioButtons(classMemberSpecification, ClassConstants.INTERNAL_ACC_PUBLIC,       publicRadioButtons);
