@@ -1,8 +1,8 @@
-/* $Id: FilterDialog.java,v 1.2 2004/08/15 12:39:30 eric Exp $
+/* $Id: FilterDialog.java,v 1.4 2005/06/11 13:13:15 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2004 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -100,7 +100,7 @@ public class FilterDialog extends JDialog
         okButtonConstraints.insets  = new Insets(4, 4, 8, 4);
 
         GridBagConstraints cancelButtonConstraints = new GridBagConstraints();
-        cancelButtonConstraints.gridwidth = GridBagConstraints.REMAINDER;;
+        cancelButtonConstraints.gridwidth = GridBagConstraints.REMAINDER;
         cancelButtonConstraints.weighty   = 1.0;
         cancelButtonConstraints.anchor    = GridBagConstraints.SOUTHEAST;
         cancelButtonConstraints.insets    = okButtonConstraints.insets;
