@@ -1,8 +1,8 @@
-/* $Id: DirectoryPump.java,v 1.2 2004/08/15 12:39:30 eric Exp $
+/* $Id: DirectoryPump.java,v 1.4 2005/08/21 20:24:12 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2004 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -45,6 +45,11 @@ public class DirectoryPump implements DataEntryPump
     public void pumpDataEntries(DataEntryReader dataEntryReader)
     throws IOException
     {
+        if (!directory.exists())
+        {
+            throw new IOException("No such file or directory");
+        }
+        
         readFiles(directory, dataEntryReader);
     }
 
