@@ -1,8 +1,8 @@
-/* $Id: MappingReader.java,v 1.8 2004/08/15 12:39:30 eric Exp $
+/* $Id: MappingReader.java,v 1.10 2005/06/11 13:13:16 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2004 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -31,12 +31,12 @@ import java.io.*;
  */
 public class MappingReader
 {
-    private String mappingFileName;
+    private File mappingFile;
 
 
-    public MappingReader(String mappingFileName)
+    public MappingReader(File mappingFile)
     {
-        this.mappingFileName = mappingFileName;
+        this.mappingFile = mappingFile;
     }
 
 
@@ -52,7 +52,7 @@ public class MappingReader
         {
             reader = new LineNumberReader(
                      new BufferedReader(
-                     new FileReader(mappingFileName)));
+                     new FileReader(mappingFile)));
 
             String className = null;
 
