@@ -1,8 +1,8 @@
-/* $Id: GotoReturnReplacer.java,v 1.6 2004/11/20 15:06:55 eric Exp $
+/* $Id: GotoReturnReplacer.java,v 1.8 2005/07/31 18:50:05 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2004 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -34,16 +34,32 @@ import proguard.classfile.instruction.*;
 public class GotoReturnReplacer implements InstructionVisitor
 {
     private CodeAttrInfoEditor codeAttrInfoEditor;
+    private InstructionVisitor extraInstructionVisitor;
 
 
     /**
      * Creates a new GotoReturnReplacer.
-     * @param codeAttrInfoEditor a code editor that can be used for
-     *                           accumulating changes to the code.
+     * @param codeAttrInfoEditor      a code editor that can be used for
+     *                                accumulating changes to the code.
      */
     public GotoReturnReplacer(CodeAttrInfoEditor codeAttrInfoEditor)
     {
-        this.codeAttrInfoEditor = codeAttrInfoEditor;
+        this(codeAttrInfoEditor, null);
+    }
+
+
+    /**
+     * Creates a new GotoReturnReplacer.
+     * @param codeAttrInfoEditor      a code editor that can be used for
+     *                                accumulating changes to the code.
+     * @param extraInstructionVisitor an optional extra visitor for all replaced
+     *                                goto instructions.
+     */
+    public GotoReturnReplacer(CodeAttrInfoEditor codeAttrInfoEditor,
+                              InstructionVisitor extraInstructionVisitor)
+    {
+        this.codeAttrInfoEditor      = codeAttrInfoEditor;
+        this.extraInstructionVisitor = extraInstructionVisitor;
     }
 
 
@@ -84,6 +100,13 @@ public class GotoReturnReplacer implements InstructionVisitor
                              new SimpleInstruction(targetInstruction.opcode);
                         codeAttrInfoEditor.replaceInstruction(offset,
                                                               returnInstruction);
+        
+                        // Visit the instruction, if required.
+                        if (extraInstructionVisitor != null)
+                        {
+                            extraInstructionVisitor.visitBranchInstruction(classFile, methodInfo, codeAttrInfo, offset, branchInstruction);
+                        }
+
                         break;
                 }
             }
