@@ -1,8 +1,8 @@
-/* $Id: NopRemover.java,v 1.5 2004/10/10 20:56:58 eric Exp $
+/* $Id: NopRemover.java,v 1.7 2005/07/31 18:50:05 eric Exp $
  *
  * ProGuard -- shrinking, optimization, and obfuscation of Java class files.
  *
- * Copyright (c) 2002-2004 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2005 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -34,16 +34,32 @@ import proguard.classfile.visitor.*;
 public class NopRemover implements InstructionVisitor
 {
     private CodeAttrInfoEditor codeAttrInfoEditor;
+    private InstructionVisitor extraInstructionVisitor;
 
 
     /**
      * Creates a new NopRemover.
-     * @param codeAttrInfoEditor a code editor that can be used for
-     *                           accumulating changes to the code.
+     * @param codeAttrInfoEditor      a code editor that can be used for
+     *                                accumulating changes to the code.
      */
     public NopRemover(CodeAttrInfoEditor codeAttrInfoEditor)
     {
-        this.codeAttrInfoEditor = codeAttrInfoEditor;
+        this(codeAttrInfoEditor, null);
+    }
+
+
+    /**
+     * Creates a new NopRemover.
+     * @param codeAttrInfoEditor      a code editor that can be used for
+     *                                accumulating changes to the code.
+     * @param extraInstructionVisitor an optional extra visitor for all removed
+     *                                nop instructions.
+     */
+    public NopRemover(CodeAttrInfoEditor codeAttrInfoEditor,
+                      InstructionVisitor extraInstructionVisitor)
+    {
+        this.codeAttrInfoEditor      = codeAttrInfoEditor;
+        this.extraInstructionVisitor = extraInstructionVisitor;
     }
 
 
@@ -63,6 +79,12 @@ public class NopRemover implements InstructionVisitor
             !codeAttrInfoEditor.isModified(offset))
         {
             codeAttrInfoEditor.deleteInstruction(offset);
+        
+            // Visit the instruction, if required.
+            if (extraInstructionVisitor != null)
+            {
+                extraInstructionVisitor.visitSimpleInstruction(classFile, methodInfo, codeAttrInfo, offset, simpleInstruction);
+            }
         }
     }
 }
